
// Animation, e.g. +kff5 or +kff360

//**********************************************************************
// Persistence of Vision Ray Tracer Scene Sample file
// File: sunriset.pov 
// Vers: 3.5
// Desc: This file uses the suns position,          
//       for sunrise, sun in due south and sunset.
// Date: 2004-16-02
// Auth: Stephen Parke based on the POVRay work SunPos() by Ingo Janssen
//       and the mathematical work of the original author Paul Schlyter.  
//       whose website has moved to 
//       http://www.stjarnhimlen.se/english.html 
//       where he provides the 'C' source for rise/set calculations.
//
//
//
// For Calgary, Canada, on our shortest day of the year,
// almost the same latitude as Greenwich, although timezone is GMT -7
// Ensure the time specified is LOCAL and Lstm is specified in _degrees_  
// Daylight savings time is NOT considered so reference standard time.
// 
//#declare Year=          2003;         // Year:  in four digits
//#declare Month=           12;         // Month: number
//#declare Day=             21;         // Day:   number
//#declare Hour=            00;         // UNUSED BY SunRiSet (Noon becomes assumed for calculation purposes)
//#declare Minute=          00;         // UNUSED BY SunRiSet
//#declare Lstm=         -7*15;         // Local time zone in degrees (east = positive, west = negative)
//#declare LAT=             51;         // Latitude in degrees.decimal, northern hemisphere = positive, southern = negative
//#declare LONG=          -114;         // Longtitude in degrees.decimal, east = positive, west is negative
//#declare Angle=   -35.0/60.0;         // Number of degrees above/below the horizon for sunrise and sunset:
//
// This angle computes the length of the day, from sunrise to sunset. 
// Sunrise/set is considered to occur when the Sun's upper limb is    
// 35 arc minutes below the horizon (this accounts for the refraction 
// of the Earth's atmosphere).                                        
//#declare Angle = -35.0/60.0;
//
// This angle computes the length of the day, from sunrise to sunset. 
// Sunrise/set is considered to occur when the Sun's upper limb is    
// 35 arc minutes below the horizon (this accounts for the refraction 
// of the Earth's atmosphere).                                        
//#declare Angle = -35.0/60.0;
//
// This Angle computes the length of the day, including civil twilight.
// Civil twilight starts/ends when the Sun's center is 6 degrees below  
// the horizon.                                                         
//#declare Angle = -6.0;
//
// This angle computes the length of the day, incl. nautical twilight.  
// Nautical twilight starts/ends when the Sun's center is 12 degrees    
// below the horizon.                                                   
//#declare Angle = -12.0;
//
// This angle computes the length of the day, incl. astronomical twilight. 
// Astronomical twilight starts/ends when the Sun's center is 18 degrees   
// below the horizon.                                                      
//#declare Angle = -18.0;
//
// Invoke the macro to set the variables and then call SunPos()
// with the resulting calculated Hours and Minutes for the 
// solar position of interest.
// 

//
//SunRiSet(Year, Month, Day, Lstm, LAT, LONG, Angle)
//
// which sets global variables in local time
//   SunRise
//   SunSouth
//   SunSet 
//

//
// Refer to SunPos() for additional setup information
//
// ====================================================== 
//
// Assumption: in the scene north is in the +Z direction, south is -Z.
// Therefore, west is -x, east is +x, and +y, is, well, UP (antigravity).
//
// Place the light source in your scene, after calling SunRiSet as above:
//   light_source {
//      SunPos(Year, Month, Day, Hour, Minute, Lstm, LAT, LONG)
//      rgb 1
//   }
//
// ======================================================
//

//--------------------------------------------------------------------------
#version 3.5;
global_settings {
                assumed_gamma 1.0
                max_trace_level 5
                }
//--------------------------------------------------------------------------


#include "SunRiSet.inc"      
#include "sunpos2.inc"
#include "colors.inc"    
#include "screen.inc"

//
// For Calgary, Canada, on our shortest day of the year,
// almost the same latitude as Greenwich, although timezone is GMT -7
// Ensure the time specified is LOCAL and Lstm is specified in _degrees_  
// Daylight savings time is NOT considered so reference standard time.
//
 
#declare Year=          2004;         // Year:  in four digits
#declare Month=            3;         // Month: number
#declare Day=             14;         // Day:   number
#declare Hour=            00;         // UNUSED BY SunRiSet (Noon becomes assumed for calculation purposes)
#declare Minute=          00;         // UNUSED BY SunRiSet
#declare Lstm=         -7*15;         // Local time zone in degrees (east = positive, west = negative)
#declare LAT=             51;         // Latitude in degrees.decimal, northern hemisphere = positive, southern = negative
#declare LONG=          -114;         // Longtitude in degrees.decimal, east = positive, west is negative
#declare Angle=   -35.0/60.0;         // Number of degrees above/below the horizon for sunrise and sunset:
 
SunRiSet(Year, Month, Day, Lstm, LAT, LONG, Angle)

//#debug concat("SunRise_H_ms  local time is: ",str(SunRise, 10,4),"\n") 
//#debug concat("SunSouth_H_ms local time is: ",str(SunSouth,10,4),"\n") 
//#debug concat("SunSet_H_ms   local time is: ",str(SunSet,  10,4),"\n\n")    

////// SAMPLE CODE FOR STILL PICTURES //////
// For still picture with sun exactly South:
// The macro calculates the hour and minute
// of SunSouth (where the sun is exactly south  
// ( -z ) or 180_compass_deg at appx Noon local time
// which varies depending upon how far across the 
// timezone the local position is.
//
// Redeclare Hour and minute based on returned value for Sun in South (noonish)
//#declare Hour = int(SunSouth);
//#declare Minute = int((SunSouth - Hour) * 60);
// If desired, any other time can happily be used, such as:
//#declare Hour = int(SunRise);                           // set starting hour to hour of sunrise
//#declare Minute = (SunRise - Hour) * 60;                // set starting minute to minutes of sunrise 
// or
//#declare Hour = int(SunSet);
//#declare Minute = int((SunSet - Hour) * 60);
//
//#debug concat("Source file Hour:Minute is      ",str(Hour,-2,0),":",str(Minute,-2,0), "\n\n")


////// SAMPLE CODE FOR ANIMATION DURING DAYLIGHT HOURS ONLY //////
// Redeclare Hour and minute based on returned value for SunRise and SunSet
//
#declare ClockMinutes = (SunSet - SunRise) * 60;        // total number of minutes of daylight between sunrise and sunset 
#declare Hour    = int(SunRise);                        // set starting hour to hour of sunrise
#declare Minutef = (SunRise - Hour) * 60;               // set starting minute to minutes of sunrise (float value)
#declare Minutef =  Minute + (clock * ClockMinutes);    // increment minutes based on animation clock value (float value)
#declare Hour    = Hour + int(Minutef/60);              // update hour to new animation value   (int value)
#declare Minute  = int(Minutef - int(Minutef/60)*60);   // update minute to balance of animation value  0..59 (int value)


// Place the light source in your scene, after calling SunRiSet as above:
//light_source { SunPos (Year, Month, +Day, Hour, Minute, Lstm, LAT, LONG)   rgb 1 }  
  light_source { SunPos2(Year, Month, +Day, Hour, Minute, Lstm, LAT, LONG)   rgb 1 }  

// +Day is later used here in the "screen.inc" function but it gets modified by the sunpos() macro
// when UTC plus longtitude changes it. Therefore use the "+" to keep the value from being returned.
// Alternatively, move the text formatting prior to the call to sunpos()   



////// My Scene //////
// Build an earth, add latitudes and longtitudes
// Give it an image map from NASA Blue Marble and rotate it so Greenwich is in front.
// URL here <http://earthobservatory.nasa.gov/Newsroom/BlueMarble/>
//
// Build a local landmark and rotate it to Calgary's latitude 
// Build a reflective sphere and align it with the sunlight  
//
// Animate the sun and the satellite from Sunrise to Sunset


#declare EarthRadius =  1.0;                    // single variable relates all objects 
#declare ER =           EarthRadius/200;        // nudge for items which need it
#declare Tower =        EarthRadius/10;         // tower is 10% of earthradius so it can be seen

   
#declare Earth = sphere  { <0, 0, 0>, EarthRadius 
        pigment {
                image_map {
                png "NASA land_shallow_topo_small.png"  // the file to read (iff/tga/gif/png/jpeg/tiff/sys)
                map_type 1                              // 0=planar, 1=spherical, 2=cylindrical, 5=torus
                //interpolate 1                           // 0=none, 1=linear, 2=bilinear, 4=normalized distance
                }
            }     
            rotate y*(-90)                              // turn the earth so that Greenwich is in front
        }   
        

#declare Latitudes = union {
        object {torus {  EarthRadius,       EarthRadius*ER }                   }   /* Equator has sun overhead March and Sept 21 */
        object {torus {  EarthRadius*0.92,  EarthRadius*ER translate  y*0.3987}}   /* Tropic of Cancer sun overhead June 21  */
        object {torus {  EarthRadius*0.92,  EarthRadius*ER translate -y*0.3987}}   /* Tropic of Capricorn sun overhead Dec 21 */
        pigment {Green} 
        }

#declare Longtitudes = union {
        object {torus {  EarthRadius,  EarthRadius*ER  rotate z*90 pigment {Green}            } }  /* Greenich */
        object {torus {  EarthRadius,  EarthRadius*ER  rotate z*90 pigment {Blue} rotate y* 15} }
        object {torus {  EarthRadius,  EarthRadius*ER  rotate z*90 pigment {Blue} rotate y* 30} }
        object {torus {  EarthRadius,  EarthRadius*ER  rotate z*90 pigment {Blue} rotate y* 45} }
        object {torus {  EarthRadius,  EarthRadius*ER  rotate z*90 pigment {Blue} rotate y* 60} }
        object {torus {  EarthRadius,  EarthRadius*ER  rotate z*90 pigment {Blue} rotate y* 75} }
        object {torus {  EarthRadius,  EarthRadius*ER  rotate z*90 pigment {Blue} rotate y* 90} }
        object {torus {  EarthRadius,  EarthRadius*ER  rotate z*90 pigment {Blue} rotate y*105} }  /* Seventh Meridian */
        object {torus {  EarthRadius,  EarthRadius*ER  rotate z*90 pigment {Blue} rotate y*120} }
        object {torus {  EarthRadius,  EarthRadius*ER  rotate z*90 pigment {Blue} rotate y*135} }
        object {torus {  EarthRadius,  EarthRadius*ER  rotate z*90 pigment {Blue} rotate y*150} }
        object {torus {  EarthRadius,  EarthRadius*ER  rotate z*90 pigment {Blue} rotate y*165} }
        }



#declare CalgaryTower = union  {
   cone     { <0, 0,   0.0      >, Tower*0.10,      <0, 0, -Tower*1.80>, Tower*0.08 } 
   cone     { <0, 0, -Tower*1.80>, Tower*0.09,      <0, 0, -Tower*1.85>, Tower*0.10 }
   cone     { <0, 0, -Tower*1.85>, Tower*0.22,      <0, 0, -Tower*2.10>, Tower*0.26 }
   cone     { <0, 0, -Tower*2.10>, Tower*0.30,      <0, 0, -Tower*2.15>, Tower*0.30 }
   cone     { <0, 0, -Tower*2.15>, Tower*0.20,      <0, 0, -Tower*2.30>, Tower*0.08 pigment {Red} }
   cone     { <0, 0, -Tower*2.30>, Tower*0.08,      <0, 0, -Tower*2.35>, Tower*0.09 pigment {Red} }
   cone     { <0, 0, -Tower*2.35>, Tower*0.05,      <0, 0, -Tower*2.45>, Tower*0.02 pigment {Red} }
   pigment {Gray50}
   translate z*(-EarthRadius)   // translate out to earth's surface
   rotate x*LAT                 // rotate from equator to the correct latitude           
   // this object is built "in front" so it doesn't have to be rotated by LONG
   }  
   

// A reflective sphere is positioned between the earth and the sun and moves with the sun
// at a distance from the surface equal to the radius of the earth, for convenience
// Sorry, no checkered planes, although I did start with one.
// This was intended to be the sun but it makes a better satellite or moon because
// it casts an eclipse shadow in the exact path of the sun
#declare SunBeamVector = vrotate(<0, 0, EarthRadius*2 >,<-Al, Az, 0>); 
sphere { SunBeamVector, EarthRadius/4   // sphere size is a fraction of the earth size
  texture {
    pigment {
      color rgb <0.8,0.8,1.0>
    }
    finish{
      diffuse 0.3
      ambient 0.0                                                         
      specular 0.6
      reflection {
        0.8
        metallic
      }
      conserve_energy
    }
  }
}  

        
union   { 
        object { Earth rotate y*LONG}           // rotate so Calgary is in front and Greenwich is easterly  
        object { Longtitudes rotate y*LONG}     // rotate so Calgary is in front and Greenwich is easterly
        object { Latitudes }                    // no need to rotate these
        object { CalgaryTower}                  // The world famous Calgary Tower landmark  
       
        // This rotation is critical - it rotates the point defined by LAT&LONG
        // onto the X-Z plane which sunpos() defines as the plane 
        // to which the sun is oriented.  If we later rotate the camera
        // by the same amount, then we just get a better view of the location.
        rotate x*(90-LAT)
        }

// Perspective (default) camera positioned slightly up, and n radii away         
/* 
If screen.inc isn't included, this works nicely:
#declare GeoCam = 
        camera {
        location  <0.0, EarthRadius, -EarthRadius*5> 
        look_at   <0.0, -EarthRadius/2,  0.0>
        right     x*image_width/image_height        
        }   
camera { GeoCam rotate x*(90-LAT)  }            // Camera in geostationary orbit somewhere south of Baja Mx.
*/                                         



#declare GeoCam =vrotate(<0, EarthRadius, -EarthRadius*6>,<(LAT+10),0 ,0 >);  // get a view position

// Set camera macro
// set location, look_at and angle
Set_Camera ( GeoCam, <0.0, -EarthRadius*3, 0.0>,50) 



// create a TrueType text shape
#declare MyTextObject =
text    {
        ttf 
        "crystal.ttf", 
        concat( str(Year,4,0), "-", str(Month,-2,0), "-", str(Day,-2,0), "  ", 
        str(Hour,-2,0),":",str(Minute,-2,0), "    Alt:",str(Al,-4,1) " Az:", str(Az,-3,0) ),
        0.01, 
        <0.0, 0.0>
        scale .07
        pigment {White}
        finish {ambient 1 diffuse 0}
        }

// Place MyTextObject in the right bottom corner
// with spacing 0.01 horizontally and 0.01 vertically.
// Confine object to visible area and scale it by 0.01 if it interacts with the scene
Screen_Object ( MyTextObject, <0,0>, < 0.01, 0.01>, true, 0.01 )

  